#!/bin/bash -x
#

usage()
{
   echo "USAGE: `basename ${0}` <product_dir> <e1|e2|e4|e5|e6> <debug|opt|prof> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
}

set_qualifiers()
{
 
    if [ -z ${basequal} ]
    then
       echo "ERROR: please specify the qualifiers"
       usage
       exit 1
    fi

    fullqual=${basequal}:${extraqual}
    qualdir=`echo ${basequal} | sed -e 's/:/-/g'`-${extraqual}
    if [ "${extraqual}" = "opt" ]
    then
	qualver=`echo ${basequal} | sed -e 's/:/_/g'`_${extraqual}
    elif [ "${extraqual}" = "debug" ]
    then
	qualver=${extraqual}_`echo ${basequal} | sed -e 's/:/_/g'`
    elif [ "${extraqual}" = "prof" ]
    then
	qualver=`echo ${basequal} | sed -e 's/:/_/g'`_${extraqual}
    else
       echo "ERROR: please specify debug, opt, or prof"
       usage
       exit 1
    fi
}

get_platform_name()
{

    OS1=`uname`
    flvr=`ups flavor`
    if [ ${OS1} = "Darwin" ]
    then
	flvr=`ups flavor -2`
    fi
    OS=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform`
    plat=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform osplat`

    if [ -z ${flvr} ]
    then
       echo "ERROR: could not determine flavor for this platform"
       echo "       something is very wrong"
       exit 1
    fi

    pkgdir=${product_dir}/${package}/${pkgver}/${flvr}-${qualdir}
    patchdir=${product_dir}/${package}/${pkgver}/patch
    blddir=${product_dir}/${package}/${pkgver}/build/${flvr}-${qualdir}
    tardir=${product_dir}/${package}/${pkgver}/tar
    sourcedir=${product_dir}/${package}/${pkgver}/source
}

select_xcc()
{
    xcc=gcc
    xcpc=g++
    if [ "${basequal}" = "e2" ]
    then
       xccdotver=4.7
       xccver=v4_7_1
    elif [ "${basequal}" = "e4" ]
    then
       xccdotver=4.8
       xccver=v4_8_1
    elif [ "${basequal}" = "e5" ]
    then
       xccdotver=4.8
       xccver=v4_8_2
    elif [ "${basequal}" = "e6" ]
    then
       xccdotver=4.9
       xccver=v4_9_1
    elif [ "${basequal}" = "e1" ]
    then
       xccdotver=4.6
       xccver=v4_6_1
    elif [ "${basequal}" = "i1" ]
    then
       xccdotver=14.0
       xccver="v14_0_2 -q i1"
       xcc=icc
       xcpc=icpc
    else
      echo "ERROR: unrecognized qualifier ${basequal}"
      usage
      exit 1
    fi
}

setup_xcc()
{
    setup ${xcc} ${xccver}
    xccshortver=`${xcc} --version | head -1 | cut -f3 -d" " | cut -f1-2 -d"."`
    xccfullver=`${xcc} --version | head -1 | cut -f3 -d" "`
    if [ ${xccshortver} = ${xccdotver} ]
    then
      echo " OK - ${xcc} version is ${xccfullver}"
    else
      echo "ERROR: wrong ${xcc} version ${xccfullver}"
      exit 1
    fi
}

die()
{
  echo "$1"; exit 2
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}
sourcetar=${5}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=cppunit
origpkgver=v1_12_1
pkgver=${origpkgver}a
ssibuildshims_version=v0_07_04
pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pkgtarname=cppunit-${pkgdotver}

get_this_dir

get_ssibuildshims

if [ "${sourcetar}" = "source" ]
then
  ${SSIBUILDSHIMS_DIR}/bin/make_source_code_tarball ${product_dir} ${package} ${pkgver}
fi

set_qualifiers

get_platform_name

select_xcc

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

if [ "${extraqual}" = "opt" ]
then
    cxxflg="-O3 -g -DNDEBUG"
elif [ "${extraqual}" = "debug" ]
then
    cxxflg="-g -O0"
elif [ "${extraqual}" = "prof" ]
then
    cxxflg="-O3 -g -DNDEBUG -fno-omit-frame-pointer"
fi

if [[ "${basequal}" == e[245] ]]
then
  cxxflg="${cxxflg} -std=c++11 -Wno-deprecated-declarations"
elif [ "${basequal}" = "e6" ]
then
  cxxflg="${cxxflg} -std=c++1y -Wno-deprecated-declarations"
elif [ "${basequal}" = "e1" ]
then
  cxxflg="${cxxflg} -std=c++0x -Wno-deprecated-declarations"
elif [ "${basequal}" = "i1" ]
then
  cxxflg="${cxxflg} -std=c++11"
fi

setup_xcc

mkdir -p ${blddir}
if [ ! -d ${blddir} ]
then
   echo "ERROR: failed to create ${blddir}"
   exit 1
fi
mkdir -p ${sourcedir}
if [ ! -d ${sourcedir} ]
then
   echo "ERROR: failed to create ${sourcedir}"
   exit 1
fi

ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`

set -x

cd ${sourcedir}
tar xf ${tardir}/${pkgtarname}.tar.gz

cd ${blddir}

# Don't check configure status -- it's buggy.
CC=${xcc} CXX=${xcpc} CXXFLAGS="${cxxflg}" ${sourcedir}/${pkgtarname}/configure --prefix=${pkgdir}
make -j ${ncore} || die "ERROR: make failed."
make install || die "ERROR: make install failed."

set +x

if [ ! -d ${pkgdir}/bin ]
then
   echo "ERROR: failed to create ${pkgdir}/bin"
   echo "       Something is very wrong"
   exit 1
fi

if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi

# ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup ${package} ${pkgver} -q ${fullqual} -z ${product_dir}
echo "cppunit libraries are in ${CPPUNIT_LIB}"

# fixing embedded path
mv ${CPPUNIT_FQ_DIR}/bin/cppunit-config ${CPPUNIT_FQ_DIR}/bin/cppunit-config.bak
cat ${CPPUNIT_FQ_DIR}/bin/cppunit-config.bak | sed 's%^prefix=.*$%prefix=${CPPUNIT_FQ_DIR}%' > ${CPPUNIT_FQ_DIR}/bin/cppunit-config
chmod +x ${CPPUNIT_FQ_DIR}/bin/cppunit-config

# this must be last 
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
fi

exit 0
